@@ -1,3 +1,8 @@
+
+#
+#
+#
+
 import StringIO
 import base64
 import httplib
@@ -77,7 +82,7 @@ class Document(object):
     def dump_draft(self):
         buf = StringIO.StringIO()
         for key in self.attr_names():
-            buf.write("%s=%s\n" % (key, self.attrs[key]))
+            buf.write("%s=%s\n" % (key, _enc(self.attrs[key])))
         if self.kwords:
             buf.write("%%VECTOR")
             for key, value in self.kwords.iteritems():
@@ -87,12 +92,13 @@ class Document(object):
             buf.write("%%SCORE\t\d\n", self.score)
         buf.write("\n")
         if self.dtexts:
-            buf.write("\n".join(self.dtexts))
+            buf.write("\n".join(_enc(self.dtexts)))
             buf.write("\n")
         if self.htexts:
             buf.write("\t")
-            buf.write("\n\t".join(self.htexts))
+            buf.write("\n\t".join(_enc(self.htexts)))
             buf.write("\n")
+
         return buf.getvalue()
 
 
@@ -182,10 +188,10 @@ class NodeResult(object):
         self.hints = hints
 
     def doc_num(self):
-        return len(docs)
+        return len(self.docs)
 
     def get_doc(self, index):
-        if index < 0 or index < self.docNum():
+        if index < 0 or index >= self.docNum():
             return None
         return self.docs[index]
 
@@ -202,15 +208,13 @@ class Transport(object):
         self.auth = None
     
     def send(self, command, headers = {}, body=None, autoContentType = True):
-        return self._send(command, headers, body)
+        # return self._send(command, headers, body)
         try:
             return self._send(command, headers, body)
         except urllib2.HTTPError, e:
-            print e
-            return e.code, None
+            return e.code, str(e)
         except Exception, e:
-            print e
-            return -1, None
+            return -1, str(e)
 
     def _send(self, command, headers = {}, body=None, autoContentType=True):
         if not self.url:
@@ -232,17 +236,15 @@ class Transport(object):
         request.add_data(body)
         for k,v in headers.iteritems():
             request.add_header(k, v)
-
         response = urllib2.urlopen(request)
         result = response.read()
         code = response.code
         
-        return code, result
+        return code, result.decode("utf-8")
 
 class Node(object):
     def __init__(self):
         self.transport = Transport()
-        self.status = 0
         self.name = None
         self.label = None
         self.dnum = -1
@@ -269,90 +271,79 @@ class Node(object):
         self.transport.auth = ":".join((name, password))
     
     def sync(self):
-        self.status = -1
-        self.status, result = self.transport.send("sync")
+        status, result = self.transport.send("sync")
         
-        return self.status == 200
+        return status == 200
 
     def optimize(self):
-        self.status = -1
-        self.status, result = self.transport.send("optimize")
+        status, result = self.transport.send("optimize")
         
-        return self.status == 200
+        return status == 200
 
     def put_doc(self, doc):
-        self.status = -1
-        self.status, result = self.transport.send("put_doc",
+        status, result = self.transport.send("put_doc",
                                                   {"Content-Type":"text/x-estraier-draft"},
                                                   doc.dump_draft())
         
-        return self.status == 200
+        return status == 200
 
     def out_doc(self, id):
-        self.status = -1
-        self.status, result = self.transport.send("out_doc",
+        status, result = self.transport.send("out_doc",
                                                   body = "id=" + str(id))
         
-        return self.status == 200
+        return status == 200
 
     def out_doc_by_uri(self, uri):
-        self.status = -1
-        body = "uri=" + cgi.escape(uri)
-        self.status, result = self.transport.send("out_doc",
+        body = "uri=" + _escape(uri)
+        status, result = self.transport.send("out_doc",
                                                   body = body)
         
-        return self.status == 200
+        return status == 200
 
     def edit_doc(self, doc):
-        self.status = -1
-        self.status, result = self.transport.send("edit_doc",
+        status, result = self.transport.send("edit_doc",
                                                   {"Content-Type":"text/x-estraier-draft"},
                                                   doc.dump_draft())
         
-        return self.status == 200
+        return status == 200
 
     def get_doc(self, id):
-        self.status = -1
         body = "id=" + str(id)
-        self.status, result = self.transport.send("get_doc", body = body)
+        status, result = self.transport.send("get_doc", body = body)
         
-        if self.status == 200:
+        if status == 200:
             return Document(result)
         return None
 
     def get_doc_by_uri(self, uri):
-        self.status = -1
-        body = "uri=" + cgi.escape(uri)
-        self.status, result = self.transport.send("get_doc", body = body)
+        body = "uri=" + _escape(uri)
+        status, result = self.transport.send("get_doc", body = body)
         
-        if self.status == 200:
+        if status == 200:
             return Document(result)
         return None
 
     def get_doc_attr(self, id, name):
-        self.status = -1
-        body = "id=" + str(id) + "&attr=" + cgi.escape(name)
-        self.status, result = self.transport.send("get_doc_attr", body = body)
+        body = "id=" + str(id) + "&attr=" + _escape(name)
+        status, result = self.transport.send("get_doc_attr", body = body)
         
-        if self.status == 200:
+        if status == 200:
             return result.strip()
         return None
 
     def get_doc_attr_by_uri(self, uri, name):
-        self.status = -1
-        body = "uri=" + cgi.escape(uri) + "&attr=" + cgi.escape(name)
-        self.status, result = self.transport.send("get_doc_attr", body = body)
+        body = "uri=" + _escape(uri) + "&attr=" + _escape(name)
+        status, result = self.transport.send("get_doc_attr", body = body)
         
-        if self.status == 200:
+        if status == 200:
             return result.strip()
         return None
 
     def etch_doc(self, id):
-        self.status = -1
         body = "id=" + str(id)
-        self.status, result = self.transport.send("etch_doc", body = body)
+        status, result = self.transport.send("etch_doc", body = body)
         
-        if self.status == 200:
+        if status == 200:
             kwords = {}
             for line in result.splitlines():
                 pair = line.split("\t")
@@ -362,11 +353,10 @@ class Node(object):
         return None
 
     def etch_doc_by_uri(self, uri):
-        self.status = -1
-        body = "uri=" + cgi.escape(uri)
-        self.status, result = self.transport.send("etch_doc", body = body)
+        body = "uri=" + _escape(uri)
+        status, result = self.transport.send("etch_doc", body = body)
         
-        if self.status == 200:
+        if status == 200:
             kwords = {}
             for line in result.splitlines():
                 pair = line.split("\t")
@@ -376,11 +366,10 @@ class Node(object):
         return None
 
     def uri_to_id(self, uri):
-        self.status = -1
-        body = "uri=" + cgi.escape(uri)
-        self.status, result = self.transport.send("uri_to_id", body = body)
+        body = "uri=" + _escape(uri)
+        status, result = self.transport.send("uri_to_id", body = body)
         
-        if self.status == 200:
+        if status == 200:
             return int(result)
         return None
     
@@ -410,10 +399,9 @@ class Node(object):
         return self.size
 
     def get_cache_usage(self):
-        self.status = -1
-        self.status, result = self.transport.send("cacheusage",
+        status, result = self.transport.send("cacheusage",
                                                   autoContentType = False)
-        if self.status != 200 or not result:
+        if status != 200 or not result:
             return -1.0
         return float(result)
 
@@ -433,11 +421,10 @@ class Node(object):
         return self.links
 
     def search(self, condition, depth = 0):
-        self.status = -1
         body = _condToQuery(condition, depth,
                                 self.wwidth, self.hwidth, self.awidth)
-        self.status, result = self.transport.send("search", body=body)
-        if self.status != 200 or not result:
+        status, result = self.transport.send("search", body=body)
+        if status != 200 or not result:
             return None
         lines = result.splitlines()
         docs = []
@@ -501,24 +488,21 @@ class Node(object):
             self.awidth = awidth
 
     def set_user(self, user, mode):
-        self.status = -1
-        body = "name=" + cgi.escape(name) + "&mode=" + str(mode)
-        self.status, result = self.transport.send("_set_user", body=body)
-        return self.status == 200
+        body = "name=" + _escape(name) + "&mode=" + str(mode)
+        status, result = self.transport.send("_set_user", body=body)
+        return status == 200
 
     def set_link(self, url, label, credit):
-        self.status = -1
-        body = "url=" + cgi.escape(url) + "&label=" + label
+        body = "url=" + _escape(url) + "&label=" + label
         if credit >= 0:
             body += "&credit=" + str(credit)
-        self.status, result = self.transport.send("_set_link", body=body)
-        return self.status == 200
+        status, result = self.transport.send("_set_link", body=body)
+        return status == 200
 
     def set_info(self):
-        self.status = -1
-        self.status, result = self.transport.send("inform",
+        status, result = self.transport.send("inform",
                                                   autoContentType = False)
-        if self.status != 200 or not result:
+        if status != 200 or not result:
             return
         lines = result.splitlines()
         line = lines.pop(0)
@@ -548,21 +532,23 @@ class Node(object):
                 currentScope[0].append(currentScope[1](line))
 
 
-SNIP_PATTERN = re.compile("[ \t\r\n\v\f]+")
-SPC_PATTERN = re.compile(" +")
+SNIP_PATTERN = re.compile(u"[ \t\r\n\v\f]+")
+SPC_PATTERN = re.compile(u" +")
 def _normalize(value):
-    value = SNIP_PATTERN.sub(" ", value)
-    value = SPC_PATTERN.sub(" ", value)
+    if type(value) == str:
+        value = value.decode("utf-8")
+    value = SNIP_PATTERN.sub(u" ", value)
+    value = SPC_PATTERN.sub(u" ", value)
     return value
 
 def _condToQuery(cond, depth, wwidth, hwidth, awidth):
     buf = []
     if cond.phrase:
-        buf.append(("phrase", cgi.escape(cond.phrase)))
+        buf.append(("phrase", _escape(cond.phrase)))
     for i in range(0, len(cond.attrs)):
-        buf.append(("attr" + str(i+1), cgi.escape(cond.attrs[i])))
+        buf.append(("attr" + str(i+1), _escape(cond.attrs[i])))
     if cond.order:
-        buf.append(("order", cgi.escape(cond.order)))
+        buf.append(("order", _escape(cond.order)))
     if cond.max >= 0:
         buf.append(("max", str(cond.max)))
     else:
@@ -571,7 +557,7 @@ def _condToQuery(cond, depth, wwidth, hwidth, awidth):
         buf.appends(("options", str(cond.options)))
     buf.append(("auxiliary", str(cond.auxiliary)))
     if cond.distinct:
-        buf.append(("distinct", cgi.escape(cond.distinct)))
+        buf.append(("distinct", _escape(cond.distinct)))
     if depth>0:
         buf.append(("depth", str(depth)))
     buf.append(("wwidth", str(wwidth)))
@@ -581,3 +567,14 @@ def _condToQuery(cond, depth, wwidth, hwidth, awidth):
     buf.append(("mask", str(cond.mask)))
     
     return "&".join(["%s=%s" % (e[0], e[1]) for e in buf])
+
+def _escape(s):
+    if type(s) == unicode:
+        s = s.encode("utf-8")
+    return cgi.escape(s)
+
+
+def _enc(s):
+    if type(s) == str:
+        s = s.decode("utf-8")
+    return s
